==> cds-services-impl-1.18.3-sources.jar: com/sap/cds/services/impl/cds/EnumAssertionHandler.java <==
/**************************************************************************
 * (C) 2019-2021 SAP SE or an SAP affiliate company. All rights reserved. *
 **************************************************************************/
package com.sap.cds.services.impl.cds;

import static com.sap.cds.services.utils.model.CdsAnnotations.ASSERT_RANGE;

import java.util.stream.Collectors;

import com.sap.cds.CdsDataProcessor;
import com.sap.cds.ql.cqn.Path;
import com.sap.cds.reflect.CdsElement;
import com.sap.cds.reflect.CdsEnumType;
import com.sap.cds.reflect.CdsType;
import com.sap.cds.services.EventContext;
import com.sap.cds.services.cds.ApplicationService;
import com.sap.cds.services.cds.CqnService;
import com.sap.cds.services.handler.EventHandler;
import com.sap.cds.services.handler.annotations.Before;
import com.sap.cds.services.handler.annotations.HandlerOrder;
import com.sap.cds.services.handler.annotations.ServiceName;
import com.sap.cds.services.impl.utils.CdsServiceUtils;
import com.sap.cds.services.impl.utils.ValidatorUtils;
import com.sap.cds.services.utils.CdsErrorStatuses;
import com.sap.cds.services.utils.OrderConstants;

@ServiceName(value = "*", type = ApplicationService.class)
public class EnumAssertionHandler implements EventHandler {

	@Before(event = { CqnService.EVENT_CREATE, CqnService.EVENT_UPDATE, CqnService.EVENT_UPSERT })
	@HandlerOrder(OrderConstants.Before.VALIDATE_FIELDS)
	public void runCheck(EventContext context) {
		CdsDataProcessor.create()
				.addValidator(EnumAssertionHandler::isEnumWithAssertRange, (path, element, elementValue) -> {
					ensureValidEnum(context, path, element, elementValue);
				}).process(CdsServiceUtils.getEntities(context), context.getTarget());
	}

	private static void ensureValidEnum(EventContext context, Path path, CdsElement element, Object elementValue) {
		if (elementValue != null) {
			CdsEnumType<?> enumElement = element.getType();
			if (!enumElement.hasValue(elementValue)) {
				String validEnumValues = enumElement.enumerals().values().stream()
						.map(e -> e.value() + " (" + e.name() + ")").collect(Collectors.joining(", "));
				ValidatorUtils.handleValidationError(context, CdsErrorStatuses.INVALID_ENUM, element.getName(),
						validEnumValues);
			}
		}
	}

	private static boolean isEnumWithAssertRange(Path path, CdsElement element, CdsType type) {
		return type.isEnum() && !Boolean.FALSE.equals(ASSERT_RANGE.getOrDefault(element));
	}

}

==> cds-services-impl-1.26.1-sources.jar: com/sap/cds/services/impl/cds/EnumAssertionHandler.java <==
/**************************************************************************
 * (C) 2019-2021 SAP SE or an SAP affiliate company. All rights reserved. *
 **************************************************************************/
package com.sap.cds.services.impl.cds;

import static com.sap.cds.services.utils.model.CdsAnnotations.ASSERT_RANGE;

import java.util.stream.Collectors;

import com.sap.cds.CdsDataProcessor;
import com.sap.cds.ql.cqn.Path;
import com.sap.cds.reflect.CdsElement;
import com.sap.cds.reflect.CdsEnumType;
import com.sap.cds.reflect.CdsType;
import com.sap.cds.services.EventContext;
import com.sap.cds.services.cds.ApplicationService;
import com.sap.cds.services.cds.CqnService;
import com.sap.cds.services.handler.EventHandler;
import com.sap.cds.services.handler.annotations.Before;
import com.sap.cds.services.handler.annotations.HandlerOrder;
import com.sap.cds.services.handler.annotations.ServiceName;
import com.sap.cds.services.impl.utils.CdsServiceUtils;
import com.sap.cds.services.impl.utils.ValidatorUtils;
import com.sap.cds.services.utils.CdsErrorStatuses;
import com.sap.cds.services.utils.OrderConstants;

@ServiceName(value = "*", type = ApplicationService.class)
public class EnumAssertionHandler implements EventHandler {

	@Before(event = { CqnService.EVENT_CREATE, CqnService.EVENT_UPDATE, CqnService.EVENT_UPSERT })
	@HandlerOrder(OrderConstants.Before.VALIDATE_FIELDS)
	public void runCheck(EventContext context) {
		CdsDataProcessor.create()
				.addValidator(EnumAssertionHandler::isEnumWithAssertRange, (path, element, elementValue) -> {
					ensureValidEnum(context, path, element, elementValue);
				}).process(CdsServiceUtils.getEntities(context), context.getTarget());
	}

	private static void ensureValidEnum(EventContext context, Path path, CdsElement element, Object elementValue) {
		if (elementValue != null) {
			CdsEnumType<?> enumElement = element.getType();
			if (!enumElement.hasValue(elementValue)) {
				String validEnumValues = enumElement.enumerals().values().stream()
						.map(e -> e.value() + " (" + e.name() + ")").collect(Collectors.joining(", "));
				ValidatorUtils.handleValidationError(context, path, element, CdsErrorStatuses.INVALID_ENUM,
						element.getName(), validEnumValues);
			}
		}
	}

	private static boolean isEnumWithAssertRange(Path path, CdsElement element, CdsType type) {
		return type.isEnum() && !Boolean.FALSE.equals(ASSERT_RANGE.getOrDefault(element));
	}

}
